/**
 * This file is part of Rubanetra.
 * Copyright (C) 2013,2014 Stefan Swerk (stefan_rubanetra@swerk.priv.at)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

import at.jku.fim.rubanetra.config.model.PortSpecification;
import at.jku.fim.rubanetra.protocol.activity.tls.TlsActivity;
import at.jku.fim.rubanetra.protocol.mapper.KrakenTransportLayerMappingFactory;
import org.junit.Test;
import org.kie.api.definition.type.FactType;

import java.util.Collection;

import static org.junit.Assert.*;

/**
 * Created by stefan on 1/15/14.
 */
public class DropboxTest extends TestBase {
    private final String dropboxFilePath = "captures/06.dropbox.open.and.close.pcapng";
    private final String spideroakFilePath = "captures/07.spideroak.open.and.close.pcapng";

    public void initializeDefaultDropboxConfig(String pcapPath) {
        initializePcapHandler(pcapPath);
        addDefaultDnsSupport();

        protoBuilder.resetCurrentProtocolConfiguration();
        protoBuilder.bindProtocol("ethernet", "ipv4").bindProtocol("ipv4", "tcp");
        protoBuilder.setPcapHandler(pcapHandler);
        protoBuilder.setPortSpecification(PortSpecification.UniversalPortSpecification.ACCEPT_ALL);
        protoBuilder.setTransportProtocolMappingStrategy(KrakenTransportLayerMappingFactory.DESTINATION_PORT_STRATEGY);
        protoBuilder.setTransportProtocolMappingFactory(transportProtocolMappingFactory);
        sysBuilder.addProtocolSetting(protoBuilder.buildProtocolConfiguration());
        setRubanetraSystemConfiguration(sysBuilder.build());
    }

    @Test
    public void spideroakTest() {
        initializeDefaultDropboxConfig(spideroakFilePath);
        while (pcapHandler.readNextPcapEntry(droolsHandler)) ;
        final Collection<?> tlsActivities = systemConfiguration.getKieSession().getObjects(object -> object.getClass().getSimpleName().equals("SpiderOakActivity"));
        assertFalse(tlsActivities.isEmpty());
        assertTrue(tlsActivities.size() == 1);
        FactType spideroakType = systemConfiguration.getKieSession().getKieBase().getFactType("defaultpkg", "SpiderOakActivity");
        TlsActivity tlsTraffic = (TlsActivity) spideroakType.get(tlsActivities.toArray()[0], "associatedTlsActivity");
        assertEquals(3, (long) tlsTraffic.getClientToServerTraffic().first()); // clientHello
        assertEquals(4, (long) tlsTraffic.getServerToClientTraffic().first()); // server Response

        assertEquals(293, (long) tlsTraffic.getClientToServerTraffic().size());
        assertEquals(578, (long) tlsTraffic.getServerToClientTraffic().size());
    }

    @Test
    public void dropboxTest() {
        initializeDefaultDropboxConfig(dropboxFilePath);
        while (pcapHandler.readNextPcapEntry(droolsHandler)) ;
        final Collection<?> tlsActivities = systemConfiguration.getKieSession().getObjects(
                object -> object.getClass().getSimpleName().equals("DropboxTlsActivity"));
//        FactType dropboxType = kieSession.getKieBase().getFactType("at.jku.fim.forensic.network.analysis","DropboxTlsActivity");
//        for(AbstractActivity a : tlsActivities) {
//            System.out.println(((DnsActivity) dropboxType.get(a, "dnsQuestion")).getDnsMessage().getQuestion().getName());
//            System.out.println(a.getCompoundFrameNumbers());
//        }
        assertTrue(tlsActivities.size() == 6);
    }
}
