/**
 * This file is part of Rubanetra.
 * Copyright (C) 2013,2014 Stefan Swerk (stefan_rubanetra@swerk.priv.at)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
import at.jku.fim.rubanetra.config.model.Port;
import at.jku.fim.rubanetra.config.model.PortSpecification;
import at.jku.fim.rubanetra.pcap.PcapActivity;
import at.jku.fim.rubanetra.pcap.PcapActivityListener;
import at.jku.fim.rubanetra.protocol.mapper.KrakenTransportLayerMappingFactory;
import org.drools.core.ObjectFilter;
import org.jnetpcap.protocol.tcpip.Tcp;
import org.junit.Test;

import java.util.Collection;

import static org.junit.Assert.*;

/**
 * Created by stefan on 1/14/14.
 */
public class HttpsTest extends TestBase {

    public void initializeDefaultHttpsSetting() {
        initializePcapHandler("./captures/08.https.rsa.snakeoil2.cap");
        protoBuilder.resetCurrentProtocolConfiguration();
        PortSpecification potSpec = new PortSpecification();
        potSpec.add(Port.create(443));
        protoBuilder.bindProtocol("ethernet", "ipv4").bindProtocol("ipv4", "tcp");
        protoBuilder.setPcapHandler(pcapHandler);
        protoBuilder.setPortSpecification(potSpec);
        protoBuilder.setTransportProtocolMappingStrategy(KrakenTransportLayerMappingFactory.DESTINATION_PORT_STRATEGY);
        protoBuilder.setTransportProtocolMappingFactory(transportProtocolMappingFactory);
        sysBuilder.addProtocolSetting(protoBuilder.buildProtocolConfiguration());
        setRubanetraSystemConfiguration(sysBuilder.build());
    }

    @Test
    public void initializeDefaultHttpsSettingTest() {
        initializeDefaultHttpsSetting();
        assertTrue(systemConfiguration.getProtocolSettings().size() == 1);
        assertNotNull(systemConfiguration.getProtocolSettings().toArray()[0]);
        assertNotNull(pcapHandler);
        assertNotNull(droolsHandler);

        final PcapActivity[] packetRecvd = {null};
        pcapHandler.readNextPcapEntry(pcapActivity -> {
            assertNotNull(pcapActivity);
            packetRecvd[0] = pcapActivity;
            assertEquals(pcapActivity.getFrameNumber(), 1);
        });
        assertNotNull(packetRecvd[0]);

        while (pcapHandler.readNextPcapEntry(pcapActivity -> {
            assertNotNull(pcapActivity);
            assertTrue(pcapActivity.getPcapPacket().hasHeader(new Tcp()));
        })) ;

    }

    @Test
    public void foundHttpsActivities() {
        initializeDefaultHttpsSetting();
        while (pcapHandler.readNextPcapEntry(droolsHandler)) ;
        final Collection<?> httpActivities = systemConfiguration.getKieSession().getObjects(new ObjectFilter() {
            @Override
            public boolean accept(Object object) {
                return object.getClass().getSimpleName().equals("HttpsActivity");
            }
        });
        for (Object o : httpActivities) {
//            System.out.println(o);
        }
        assertFalse(httpActivities.isEmpty());
    }
}
