/**
 * This file is part of Rubanetra.
 * Copyright (C) 2013,2014 Stefan Swerk (stefan_rubanetra@swerk.priv.at)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
import at.jku.fim.rubanetra.config.model.PortSpecification;
import at.jku.fim.rubanetra.protocol.activity.tcp.TcpActivity;
import at.jku.fim.rubanetra.protocol.mapper.KrakenTransportLayerMappingFactory;
import org.drools.core.ObjectFilter;
import org.junit.Test;

import java.util.Collection;

import static org.junit.Assert.assertFalse;

/**
 * Created by stefan on 1/15/14.
 */
public class SshTest extends TestBase {
    private final String sshFilePath = "captures/09.ssh.pcap";

    public void initializeDefaultSshConfig(String pcapPath) {
        initializePcapHandler(pcapPath);
        protoBuilder.resetCurrentProtocolConfiguration();
        protoBuilder.bindProtocol("ethernet", "ipv4").bindProtocol("ipv4", "tcp");
        protoBuilder.setPcapHandler(pcapHandler);
        protoBuilder.setPortSpecification(PortSpecification.UniversalPortSpecification.ACCEPT_ALL);
        protoBuilder.setTransportProtocolMappingStrategy(KrakenTransportLayerMappingFactory.DESTINATION_PORT_STRATEGY);
        protoBuilder.setTransportProtocolMappingFactory(transportProtocolMappingFactory);
        sysBuilder.addProtocolSetting(protoBuilder.buildProtocolConfiguration());
        setRubanetraSystemConfiguration(sysBuilder.build());
    }

    @Test
    public void sshTest() {
        initializeDefaultSshConfig(sshFilePath);
        while (pcapHandler.readNextPcapEntry(droolsHandler)) ;
        final Collection<?> sshActivities = systemConfiguration.getKieSession().getObjects(new ObjectFilter() {
            @Override
            public boolean accept(Object object) {
                return object instanceof TcpActivity;
            }
        });
        for (Object o : sshActivities) {
            TcpActivity a = (TcpActivity) o;
//            System.out.println(a.payloadHexString());
        }
        assertFalse(sshActivities.isEmpty());
    }
}
