/**
 * This file is part of Rubanetra.
 * Copyright (C) 2013,2014 Stefan Swerk (stefan_rubanetra@swerk.priv.at)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
import at.jku.fim.rubanetra.config.model.PortSpecification;
import at.jku.fim.rubanetra.config.model.RubanetraSystemConfiguration;
import at.jku.fim.rubanetra.config.model.impl.KrakenProtocolConfigurationBuilderImpl;
import at.jku.fim.rubanetra.config.model.impl.RubanetraSystemConfigurationBuilder;
import at.jku.fim.rubanetra.drools.DroolsKrakenProtocolHandler;
import at.jku.fim.rubanetra.pcap.PcapActivityListener;
import at.jku.fim.rubanetra.pcap.PcapHandler;
import at.jku.fim.rubanetra.protocol.mapper.KrakenTransportLayerMappingFactory;
import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.io.IOException;
import java.net.URISyntaxException;
import java.net.URL;

/**
 * Created by stefan on 1/14/14.
 */
public class TestBase {
    protected final Logger log = LoggerFactory.getLogger(getClass());
    protected RubanetraSystemConfigurationBuilder sysBuilder;
    protected KrakenProtocolConfigurationBuilderImpl protoBuilder;
    protected RubanetraSystemConfiguration systemConfiguration;
    protected DroolsKrakenProtocolHandler droolsHandler;
    protected File pcapFile;
    protected KrakenTransportLayerMappingFactory transportProtocolMappingFactory;
    protected PcapHandler<PcapActivityListener> pcapHandler;

    @Before
    public void initialize() throws URISyntaxException {
        sysBuilder = RubanetraSystemConfigurationBuilder.create();
        sysBuilder.setOutputFormat(RubanetraSystemConfiguration.OutputFormat.NOP.name()).setOutputStream(System.out);

        protoBuilder = new KrakenProtocolConfigurationBuilderImpl();
        transportProtocolMappingFactory = KrakenTransportLayerMappingFactory.createDefaultKrakenTransportLayerMappingFactory();
    }

    @After
    public void close() throws IOException {
        droolsHandler.close();
    }

    protected void setRubanetraSystemConfiguration(RubanetraSystemConfiguration systemConfiguration) {
        this.systemConfiguration = systemConfiguration;
        droolsHandler = new DroolsKrakenProtocolHandler(systemConfiguration);
    }

    public void initializePcapHandler(String... filePaths) {
        for (String filePath : filePaths) {
            URL fileUrl = ClassLoader.getSystemResource(filePath);
            try {
                pcapFile = new File(fileUrl.toURI());
            } catch (URISyntaxException e) {
                Assert.fail();
            }
            sysBuilder.addInputPath(filePath);
            pcapHandler = sysBuilder.createPcapHandler();
        }
    }

    public void addDefaultDnsSupport() {
        protoBuilder.resetCurrentProtocolConfiguration();
//        protoBuilder.bindProtocol("ethernet", "ipv4").bindProtocol("ipv4", "udp").bindProtocol("ipv4", "tcp").bindProtocol("udp", "dns").bindProtocol("tcp", "dns");
        protoBuilder.bindProtocol("ethernet", "ipv4").bindProtocol("ipv4", "udp").bindProtocol("udp", "dns");
        protoBuilder.setPcapHandler(pcapHandler);
        protoBuilder.setPortSpecification(PortSpecification.UniversalPortSpecification.ACCEPT_ALL);
        protoBuilder.setTransportProtocolMappingStrategy(KrakenTransportLayerMappingFactory.DESTINATION_PORT_STRATEGY);
        protoBuilder.setTransportProtocolMappingFactory(transportProtocolMappingFactory);
        sysBuilder.addProtocolSetting(protoBuilder.buildProtocolConfiguration());

        protoBuilder.resetCurrentProtocolConfiguration();
        protoBuilder.bindProtocol("ethernet", "ipv4").bindProtocol("ipv4", "tcp").bindProtocol("tcp", "dns");
        protoBuilder.setPcapHandler(pcapHandler);
        protoBuilder.setPortSpecification(PortSpecification.UniversalPortSpecification.ACCEPT_ALL);
        protoBuilder.setTransportProtocolMappingStrategy(KrakenTransportLayerMappingFactory.DESTINATION_PORT_STRATEGY);
        protoBuilder.setTransportProtocolMappingFactory(transportProtocolMappingFactory);
        sysBuilder.addProtocolSetting(protoBuilder.buildProtocolConfiguration());
    }

    @After
    public void cleanUp() throws IOException {
        if (this.systemConfiguration != null && this.systemConfiguration.getKieSession() != null)
            this.systemConfiguration.getKieSession().dispose();
        if (this.pcapHandler != null)
            this.pcapHandler.close();
        this.droolsHandler = null;
        this.pcapHandler = null;
        this.protoBuilder = null;
        this.sysBuilder = null;
        this.systemConfiguration = null;
    }
}
